@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
 
 from xml.dom import minidom
+from _pytest.main import EXIT_NOTESTSCOLLECTED
 import py, sys, os
 from _pytest.junitxml import LogXML
+import pytest
+
 
 def runandparse(testdir, *args):
     resultpath = testdir.tmpdir.join("junit.xml")
@@ -44,6 +47,10 @@ class TestPython:
     def test_timing_function(self, testdir):
         testdir.makepyfile("""
             import time, pytest
+            def setup_module():
+                time.sleep(0.01)
+            def teardown_module():
+                time.sleep(0.01)
             def test_sleep():
                 time.sleep(0.01)
         """)
@@ -51,7 +58,7 @@ class TestPython:
         node = dom.getElementsByTagName("testsuite")[0]
         tnode = node.getElementsByTagName("testcase")[0]
         val = tnode.getAttributeNode("time").value
-        assert float(val) >= 0.001
+        assert round(float(val), 2) >= 0.03
 
     def test_setup_error(self, testdir):
         testdir.makepyfile("""
@@ -66,6 +73,8 @@ class TestPython:
         assert_attr(node, errors=1, tests=0)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_setup_error.py",
+            line="2",
             classname="test_setup_error",
             name="test_function")
         fnode = tnode.getElementsByTagName("error")[0]
@@ -84,6 +93,8 @@ class TestPython:
         assert_attr(node, skips=1)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_skip_contains_name_reason.py",
+            line="1",
             classname="test_skip_contains_name_reason",
             name="test_skip")
         snode = tnode.getElementsByTagName("skipped")[0]
@@ -104,6 +115,8 @@ class TestPython:
         assert_attr(node, failures=1)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_classname_instance.py",
+            line="1",
             classname="test_classname_instance.TestClass",
             name="test_method")
 
@@ -116,6 +129,8 @@ class TestPython:
         assert_attr(node, failures=1)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file=os.path.join("sub", "test_hello.py"),
+            line="0",
             classname="sub.test_hello",
             name="test_func")
 
@@ -147,6 +162,8 @@ class TestPython:
         assert_attr(node, failures=1, tests=1)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_failure_function.py",
+            line="1",
             classname="test_failure_function",
             name="test_fail")
         fnode = tnode.getElementsByTagName("failure")[0]
@@ -189,6 +206,8 @@ class TestPython:
 
             tnode = node.getElementsByTagName("testcase")[index]
             assert_attr(tnode,
+                file="test_failure_escape.py",
+                line="1",
                 classname="test_failure_escape",
                 name="test_func[%s]" % char)
             sysout = tnode.getElementsByTagName('system-out')[0]
@@ -210,10 +229,14 @@ class TestPython:
         assert_attr(node, failures=1, tests=2)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_junit_prefixing.py",
+            line="0",
             classname="xyz.test_junit_prefixing",
             name="test_func")
         tnode = node.getElementsByTagName("testcase")[1]
         assert_attr(tnode,
+            file="test_junit_prefixing.py",
+            line="3",
             classname="xyz.test_junit_prefixing."
                       "TestHello",
             name="test_hello")
@@ -230,6 +253,8 @@ class TestPython:
         assert_attr(node, skips=1, tests=0)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_xfailure_function.py",
+            line="1",
             classname="test_xfailure_function",
             name="test_xfail")
         fnode = tnode.getElementsByTagName("skipped")[0]
@@ -249,6 +274,8 @@ class TestPython:
         assert_attr(node, skips=1, tests=0)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_xfailure_xpass.py",
+            line="1",
             classname="test_xfailure_xpass",
             name="test_xpass")
         fnode = tnode.getElementsByTagName("skipped")[0]
@@ -263,8 +290,10 @@ class TestPython:
         assert_attr(node, errors=1, tests=0)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_collect_error.py",
             #classname="test_collect_error",
             name="test_collect_error")
+        assert tnode.getAttributeNode("line") is None
         fnode = tnode.getElementsByTagName("error")[0]
         assert_attr(fnode, message="collection failure")
         assert "SyntaxError" in fnode.toxml()
@@ -272,13 +301,15 @@ class TestPython:
     def test_collect_skipped(self, testdir):
         testdir.makepyfile("import pytest; pytest.skip('xyz')")
         result, dom = runandparse(testdir)
-        assert not result.ret
+        assert result.ret == EXIT_NOTESTSCOLLECTED
         node = dom.getElementsByTagName("testsuite")[0]
         assert_attr(node, skips=1, tests=0)
         tnode = node.getElementsByTagName("testcase")[0]
         assert_attr(tnode,
+            file="test_collect_skipped.py",
             #classname="test_collect_error",
             name="test_collect_skipped")
+        assert tnode.getAttributeNode("line") is None # py.test doesn't give us a line here.
         fnode = tnode.getElementsByTagName("skipped")[0]
         assert_attr(fnode, message="collection skipped")
 
@@ -506,6 +537,7 @@ def test_unicode_issue368(testdir):
         longrepr = ustr
         sections = []
         nodeid = "something"
+        location = 'tests/filename.py', 42, 'TestClass.method'
     report = Report()
 
     # hopefully this is not too brittle ...
@@ -524,3 +556,37 @@ def test_unicode_issue368(testdir):
     log.pytest_sessionfinish()
 
 
+def test_record_property(testdir):
+    testdir.makepyfile("""
+        def test_record(record_xml_property):
+            record_xml_property("foo", "<1");
+    """)
+    result, dom = runandparse(testdir, '-rw')
+    node = dom.getElementsByTagName("testsuite")[0]
+    tnode = node.getElementsByTagName("testcase")[0]
+    psnode = tnode.getElementsByTagName('properties')[0]
+    pnode = psnode.getElementsByTagName('property')[0]
+    assert_attr(pnode, name="foo", value="<1")
+    result.stdout.fnmatch_lines('*C3*test_record_property.py*experimental*')
+
+
+def test_random_report_log_xdist(testdir):
+    """xdist calls pytest_runtest_logreport as they are executed by the slaves,
+    with nodes from several nodes overlapping, so junitxml must cope with that
+    to produce correct reports. #1064
+    """
+    pytest.importorskip('xdist')
+    testdir.makepyfile("""
+        import pytest, time
+        @pytest.mark.parametrize('i', list(range(30)))
+        def test_x(i):
+            assert i != 22
+    """)
+    _, dom = runandparse(testdir, '-n2')
+    suite_node = dom.getElementsByTagName("testsuite")[0]
+    failed = []
+    for case_node in suite_node.getElementsByTagName("testcase"):
+        if case_node.getElementsByTagName('failure'):
+            failed.append(case_node.getAttributeNode('name').value)
+
+    assert failed == ['test_x[22]']
